var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asArray, asJSON, asObject, asOptional, asString } from 'cleaners';
import { makeReactNativeDisklet } from 'disklet';
import { NativeModules, Platform } from 'react-native';
const { AbcCoreJsUi } = NativeModules;
const disklet = makeReactNativeDisklet();
function createKeyWithUsername(username) {
    return username + '___key_loginkey';
}
const asFingerprintFile = asJSON(asObject({
    enabledUsers: asOptional(asArray(asString), []),
    disabledUsers: asOptional(asArray(asString), [])
}));
export function isTouchEnabled(username) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield loadFingerprintFile();
        const supported = yield supportsTouchId();
        return supported && file.enabledUsers.includes(username);
    });
}
export function isTouchDisabled(username) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield loadFingerprintFile();
        const supported = yield supportsTouchId();
        return !supported || file.disabledUsers.includes(username);
    });
}
export function supportsTouchId() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!AbcCoreJsUi) {
            console.warn('AbcCoreJsUi  is unavailable');
            return false;
        }
        const out = yield AbcCoreJsUi.supportsTouchId();
        return !!out;
    });
}
export function enableTouchId(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield loadFingerprintFile();
        const supported = yield supportsTouchId();
        if (!supported)
            throw new Error('TouchIdNotSupportedError');
        const { username, loginKey } = account;
        const loginKeyKey = createKeyWithUsername(username);
        yield AbcCoreJsUi.setKeychainString(loginKey, loginKeyKey);
        // Update the file:
        if (!file.enabledUsers.includes(username)) {
            file.enabledUsers = [...file.enabledUsers, username];
        }
        if (file.disabledUsers.includes(username)) {
            file.disabledUsers = file.disabledUsers.filter(item => item !== username);
        }
        saveFingerprintFile(file);
    });
}
export function disableTouchId(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield loadFingerprintFile();
        const supported = yield supportsTouchId();
        if (!supported)
            return; // throw new Error('TouchIdNotSupportedError')
        const { username } = account;
        const loginKeyKey = createKeyWithUsername(username);
        yield AbcCoreJsUi.clearKeychain(loginKeyKey);
        // Update the file:
        if (!file.disabledUsers.includes(username)) {
            file.disabledUsers = [...file.disabledUsers, username];
        }
        if (file.enabledUsers.includes(username)) {
            file.enabledUsers = file.enabledUsers.filter(item => item !== username);
        }
        yield saveFingerprintFile(file);
    });
}
export function getSupportedBiometryType() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const biometryType = yield AbcCoreJsUi.getSupportedBiometryType();
            if (biometryType) {
                return biometryType;
            }
            return null;
        }
        catch (error) {
            console.log(error);
            return null;
        }
    });
}
/**
 * Looks up the stored biometric secret for a user.
 * Returns undefined if there is no secret, or if the user denies the request.
 */
export function getLoginKey(username, promptString, fallbackString) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield loadFingerprintFile();
        const supported = yield supportsTouchId();
        if (!supported ||
            !file.enabledUsers.includes(username) ||
            file.disabledUsers.includes(username)) {
            return;
        }
        const loginKeyKey = createKeyWithUsername(username);
        if (Platform.OS === 'ios') {
            const loginKey = yield AbcCoreJsUi.getKeychainString(loginKeyKey);
            if (typeof loginKey !== 'string' || loginKey.length <= 10) {
                console.log('No valid loginKey for TouchID');
                return;
            }
            console.log('loginKey valid. Launching TouchID modal...');
            const success = yield AbcCoreJsUi.authenticateTouchID(promptString, fallbackString);
            if (success)
                return loginKey;
            console.log('Failed to authenticate TouchID');
        }
        else if (Platform.OS === 'android') {
            return AbcCoreJsUi.getKeychainStringWithFingerprint(loginKeyKey, promptString).catch((error) => console.log(error)); // showError?
        }
    });
}
export function loadFingerprintFile() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const json = yield disklet.getText('fingerprint.json');
            return asFingerprintFile(json);
        }
        catch (error) {
            return { enabledUsers: [], disabledUsers: [] };
        }
    });
}
function saveFingerprintFile(file) {
    return __awaiter(this, void 0, void 0, function* () {
        const text = JSON.stringify(file);
        yield disklet.setText('fingerprint.json', text);
    });
}
